@@ -17,18 +17,20 @@
 #ifndef DSSIMODULE_H
 #define DSSIMODULE_H
 
-#include "PluginModule.h"
+#include "OmModule.h"
 
 namespace OmGtk {
 	
 	
-class DSSIModule : public PluginModule
+class DSSIModule : public OmModule
 {
 public:
 	DSSIModule(OmPatchBayArea* patch_bay, NodeModel* nm, PatchController* patch_controller);
 	virtual ~DSSIModule() {}
 
-	void show_dialog();
+	void on_double_click();
+	bool attempt_to_show_gui();
+	void show_gui();
 };
 
 
