@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -23,6 +23,9 @@
 namespace OmGtk {
 	
 
+// FIXME: awful code duplication with NewSubpatchWindow
+
+	
 /** 'New Patch' Window.
  *
  * Loaded by libglade as a derived object.
@@ -35,11 +38,13 @@ class NewPatchWindow : public Gtk::Window
 public:
 	NewPatchWindow(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
 
+	void name_changed();
 	void cancel_clicked();
 	void ok_clicked();
 
 private:
 	Gtk::Entry*      m_name_entry;
+	Gtk::Label*      m_message_label;
 	Gtk::SpinButton* m_poly_spinbutton;
 	Gtk::Button*     m_cancel_button;
 	Gtk::Button*     m_ok_button;
