@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -19,7 +19,11 @@
 
 namespace OmGtk
 {
-	OmGtkApp*          app;
-	OmGtkStateManager* state_manager;
-	Controller*        controller;
+	OmGtkApp*       app             = 0;
+	StateManager*   state_manager   = 0;
+	Controller*     controller      = 0;
+	GladeFactory*   glade_factory   = 0;
+#ifdef HAVE_LASH
+	LashController* lash_controller = 0;
+#endif
 }
