@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -18,6 +18,8 @@
 #ifndef OMGTK_H
 #define OMGTK_H
 
+#include "config.h"
+
 /** \defgroup OmGtk Modular GTK Om client
  */
 
@@ -29,12 +31,19 @@
 namespace OmGtk {
 	
 class OmGtkApp;
-class OmGtkStateManager;
+class StateManager;
 class Controller;
-	
-extern OmGtkApp*          app;
-extern OmGtkStateManager* state_manager;
-extern Controller*        controller;
+class GladeFactory;
+class LashController;
+
+extern OmGtkApp*     app;
+extern StateManager* state_manager;
+extern Controller*   controller;
+extern GladeFactory* glade_factory;
+
+#ifdef HAVE_LASH
+extern LashController* lash_controller;
+#endif // HAVE_LASH
 
 } // namespace OmGtk
 
