@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -27,20 +27,37 @@ using PatchBay::Port;
 
 namespace OmGtk {
 	
+class OmModule;
+class PatchController;
+
 
 class OmPatchBayArea : public PatchBay::PatchBayArea
 {
 public:
-	OmPatchBayArea(const string& patch_name, int width, int height);
+	OmPatchBayArea(PatchController* controller, const string& patch_name, int width, int height);
 	
+	OmModule* find_module(const string& name)
+		{ return (OmModule*)PatchBayArea::find_module(name); }
+
 	void connect(const Port* const port1, const Port* const port2);
 	void disconnect(const Port* const port1, const Port* const port2);
 	
-	void status_message(const string& msg);
+	bool canvas_event(GdkEvent* event);
+
+	void show_menu(GdkEvent* event) { m_menu.popup(event->button.button, event->button.time); }
+	
+	void menu_load_plugin();
+	void menu_load_subpatch();
+	void menu_create_subpatch();
 
 private:
-	string m_patch_path;
-	string m_patch_name;
+	PatchController* m_patch_controller;
+	string           m_patch_path;
+	string           m_patch_name;
+	int              m_last_click_x;
+	int              m_last_click_y;
+	
+	Gtk::Menu m_menu;
 };
 
 
