@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -21,7 +21,7 @@
 namespace Om {
 
 
-ActivateEvent::ActivateEvent(Request request)
+ActivateEvent::ActivateEvent(Request* request)
 : SlowEvent(request)
 {
 }
@@ -48,9 +48,9 @@ void
 ActivateEvent::post_process()
 {
 	if (om != NULL)
-		m_request.respond_ok();
+		m_request->respond_ok();
 	else
-		m_request.respond_error("Not ready to activate yet.");
+		m_request->respond_error("Not ready to activate yet.");
 }
 
 
