@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -29,7 +29,7 @@ namespace Om {
 class ActivateEvent : public SlowEvent
 {
 public:
-	ActivateEvent(Request request);
+	ActivateEvent(Request* request);
 	
 	void prepare();
 	void execute(uint sample_offset);
