@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -19,6 +19,7 @@
 
 #include "SlowEvent.h"
 #include <string>
+#include "List.h"
 
 namespace Om {
 
@@ -26,7 +27,7 @@ namespace Om {
 class Patch;
 class Node;
 class TreeNode;
-class PluginInfo;
+class Plugin;
 
 using std::string;
 
@@ -38,21 +39,24 @@ using std::string;
 class AddNodeEvent : public SlowEvent
 {
 public:
-	AddNodeEvent(Request request, const string& path, PluginInfo* pi, bool poly);
-	
+	AddNodeEvent(Request* request, const string& path, Plugin* pi, bool poly);
+	~AddNodeEvent();
+
 	void prepare();
 	void execute(uint sample_offset);
 	void post_process();
 
 private:
-	string      m_patch_name;
-	string      m_path;
-	PluginInfo* m_plugin_info;
-	bool        m_poly;
-	Patch*      m_patch;
-	Node*       m_node;
-	TreeNode*   m_tree_node;
-	bool        m_node_already_exists;
+	string       m_patch_name;
+	string       m_path;
+	Plugin*  m_plugin;
+	bool         m_poly;
+	Patch*       m_patch;
+	Node*        m_node;
+	TreeNode*    m_tree_node;
+	List<Node*>* m_process_order; // Patch's new process order
+	bool         m_node_already_exists;
+	bool         m_acquired_mutex;
 };
 
 
