@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -30,7 +30,7 @@ namespace Om {
 class DSSIConfigureEvent : public SlowEvent
 {
 public:
-	DSSIConfigureEvent(Request request, const string& node_path, const string& key, const string& val);
+	DSSIConfigureEvent(Request* request, const string& node_path, const string& key, const string& val);
 
 	void prepare();
 	void execute(uint sample_offset);
