@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -25,14 +25,14 @@ namespace Om {
 
 /** A control change event for a DSSI plugin.
  *
- * This does essentially the same thing as a SetControlEvent.
+ * This does essentially the same thing as a SetPortValueEvent.
  *
  * \ingroup engine
  */
 class DSSIControlEvent : public SlowEvent
 {
 public:
-	DSSIControlEvent(Request request, const string& node_path, int port_num, sample val);
+	DSSIControlEvent(Request* request, const string& node_path, int port_num, sample val);
 
 	void prepare();
 	void execute(uint sample_offset);
