@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -21,7 +21,7 @@
 namespace Om {
 
 
-DeactivateEvent::DeactivateEvent(Request request)
+DeactivateEvent::DeactivateEvent(Request* request)
 : SlowEvent(request)
 {
 }
@@ -44,7 +44,7 @@ DeactivateEvent::execute(uint sample_offset)
 void
 DeactivateEvent::post_process()
 {
-	m_request.respond_ok();
+	m_request->respond_ok();
 	om->deactivate();
 }
 
