@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -22,7 +22,7 @@
 namespace Om {
 
 
-DisableEvent::DisableEvent(Request request)
+DisableEvent::DisableEvent(Request* request)
 : SlowEvent(request)
 {
 }
@@ -45,9 +45,9 @@ DisableEvent::execute(uint sample_offset)
 void
 DisableEvent::post_process()
 {
-	m_request.respond_ok();
+	m_request->respond_ok();
 	om->disable();
-	om->osc_sender()->send_engine_disable(m_request);
+	om->osc_sender()->send_engine_disable();
 }
 
 
