@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,7 @@ namespace Om {
 class DisablePatchEvent : public SlowEvent
 {
 public:
-	DisablePatchEvent(Request request, const string& patch_path);
+	DisablePatchEvent(Request* request, const string& patch_path);
 	
 	void prepare();
 	void execute(uint sample_offset);
