@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,7 @@ namespace Om {
 class EnablePatchEvent : public SlowEvent
 {
 public:
-	EnablePatchEvent(Request request, const string& patch_path);
+	EnablePatchEvent(Request* request, const string& patch_path);
 	
 	void prepare();
 	void execute(uint sample_offset);
@@ -44,6 +44,7 @@ private:
 	string       m_patch_path;
 	Patch*       m_patch;
 	List<Node*>* m_process_order; // Patch's new process order
+	bool         m_acquired_mutex;
 };
 
 
