@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -25,7 +25,7 @@ namespace Om {
 // issue.  Any latency whatsoever in getting the timestamp is bad
 // bad bad
 
-Event::Event(Request request)
+Event::Event(Request* request)
 : m_request(request),
   m_executed(false)
 {
