@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -16,12 +16,12 @@
 
 #include "LoadPluginsEvent.h"
 #include "Om.h"
-#include "PluginFactory.h"
+#include "NodeFactory.h"
 
 namespace Om {
 
 
-LoadPluginsEvent::LoadPluginsEvent(Request request)
+LoadPluginsEvent::LoadPluginsEvent(Request* request)
 : SlowEvent(request)
 {
 }
@@ -44,8 +44,9 @@ LoadPluginsEvent::execute(uint sample_offset)
 void
 LoadPluginsEvent::post_process()
 {
-	m_request.respond_ok();
-	om->plugin_factory()->load_plugins();
+	// Why is this done here and not in prepare()???
+	om->node_factory()->load_plugins();
+	m_request->respond_ok();
 }
 
 
