@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -22,14 +22,14 @@
 namespace Om {
 
 
-/** Loads all plugins into the internal plugin database (in PluginFactory).
+/** Loads all plugins into the internal plugin database (in NodeFactory).
  *
  * \ingroup engine
  */
 class LoadPluginsEvent : public SlowEvent
 {
 public:
-	LoadPluginsEvent(Request request);
+	LoadPluginsEvent(Request* request);
 	
 	void prepare();
 	void execute(uint sample_offset);
