@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -17,6 +17,8 @@
 #ifndef MAIDOBJECT_H
 #define MAIDOBJECT_H
 
+#include <cstdlib>
+
 namespace Om {
 
 	
@@ -33,7 +35,9 @@ public:
 	static unsigned object_count() { return m_count; }
 
 private:
-	MaidObject(const MaidObject& copy) { throw; } // disallow copies
+	// Prevent copies
+	MaidObject(const MaidObject& copy)            { exit(EXIT_FAILURE); }
+	MaidObject& operator=(const MaidObject& copy) { exit(EXIT_FAILURE); }
 
 	/** Count of existing MaidObjects.  Used to check for leaks. */
 	static unsigned m_count;
