@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -32,7 +32,7 @@ namespace Om {
 class MidiControlEvent : public Event
 {
 public:
-	MidiControlEvent(Request request, Patch* patch, int param, int value);
+	MidiControlEvent(Request* request, Patch* patch, int param, int value);
 	
 	void execute(uint sample_offset);
 
