@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -32,10 +32,11 @@ class Patch;
 class NoteOffEvent : public Event
 {
 public:
-	NoteOffEvent(Request request, Patch* patch, int note_num);
-	NoteOffEvent(Request request, const string& patch_path, int note_num);
+	NoteOffEvent(Request* request, Patch* patch, int note_num);
+	NoteOffEvent(Request* request, const string& patch_path, int note_num);
 	
 	void execute(uint sample_offset);
+	void post_process();
 
 private:
 	Patch* m_patch;
