@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -32,16 +32,18 @@ class Patch;
 class NoteOnEvent : public Event
 {
 public:
-	NoteOnEvent(Request request, Patch* patch, int note_num, int velocity);
-	NoteOnEvent(Request request, const string& patch_path, int note_num, int velocity);
+	NoteOnEvent(Request* request, Patch* patch, int note_num, int velocity);
+	NoteOnEvent(Request* request, const string& patch_path, int note_num, int velocity);
 	
 	void execute(uint sample_offset);
+	void post_process();
 
 private:
 	Patch* m_patch;
 	string m_patch_path;
 	int    m_note_num;
 	int    m_velocity;
+	bool   m_is_osc_triggered;
 };
 
 
