@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -16,6 +16,9 @@
 
 
 #include "OmApp.h"
+#ifdef HAVE_LASH
+#include "LashDriver.h"
+#endif
 
 /** Om namespace.
  *
@@ -24,6 +27,10 @@
  * exceptions, some classes in src/common are used by clients but are part
  * of the Om namespace.
  */
-namespace Om {
-	OmApp* om;
+namespace Om
+{
+	OmApp* om = 0;
+#ifdef HAVE_LASH
+	LashDriver* lash_driver = 0;
+#endif
 }
