@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -18,6 +18,7 @@
 #ifndef OM_H
 #define OM_H
 
+#include "config.h"
 #include "OmApp.h"
 
 
@@ -30,6 +31,11 @@
  */
 namespace Om
 {
+#ifdef HAVE_LASH
+	class LashDriver;
+	extern LashDriver* lash_driver;
+#endif
+
 	extern OmApp* om;
 }
 
