@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -24,7 +24,7 @@ namespace Om {
 OutputPort::OutputPort(Node* node, const string& name, uint index, uint poly, PortInfo* port_info, size_t buffer_size)
 : Port(node, name, index, poly, port_info, buffer_size)
 {
-	assert(!is_input());
+	assert(is_output() && !is_input());
 }
 
 
