@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -18,16 +18,15 @@
 #include "Om.h"
 #include "OmApp.h"
 #include "OSCReceiver.h"
-#include <iostream>
+#include "OSCSender.h"
 
 namespace Om {
 
 
-RegisterClientEvent::RegisterClientEvent(Request request, const string& host, const string& port)
+RegisterClientEvent::RegisterClientEvent(Request* request, const string& url)
 : SlowEvent(request),
-  m_host(host),
-  m_port(port),
-  m_success(false)
+  m_url(url),
+  m_client_id(-1)
 {
 }
 
@@ -35,7 +34,7 @@ RegisterClientEvent::RegisterClientEvent(Request request, const string& host, co
 void
 RegisterClientEvent::prepare()
 {
-	m_success = om->osc_receiver()->register_client(m_host, m_port);
+	m_client_id = om->osc_receiver()->register_client(m_url);
 	
 	SlowEvent::prepare();
 }
@@ -52,10 +51,12 @@ RegisterClientEvent::execute(uint sample_offset)
 void
 RegisterClientEvent::post_process()
 {
-	if (m_success)
-		m_request.respond_ok();
+	if (m_client_id >= 0)
+		m_request->respond_ok();
 	else
-		m_request.respond_error("Unable to register client");
+		m_request->respond_error("Unable to register client");
+
+	//om->osc_sender()->send_client_registration(m_url, m_client_id);
 }
 
 
