@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -29,16 +29,15 @@ namespace Om {
 class RegisterClientEvent : public SlowEvent
 {
 public:
-	RegisterClientEvent(Request request, const string& host, const string& port);
-	
+	RegisterClientEvent(Request* request, const string& url);
+
 	void prepare();
 	void execute(uint sample_offset);
 	void post_process();
 
 private:
-	string m_host;
-	string m_port;
-	bool   m_success;
+	string m_url;
+	int    m_client_id;
 };
 
 
