@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -17,16 +17,19 @@
 #ifndef REQUEST_H
 #define REQUEST_H
 
+#include <cstdlib>
 #include <string>
 #include <lo/lo.h>
 
 using std::string;
 
+
+#include <iostream>
+using std::cerr; using std::endl;
+
 namespace Om {
 
 #define NOBODY NULL
-#define NIL_REQUEST Request(NOBODY, 0)
-
 
 /** Represents an OSC request from some client.
  *
@@ -36,11 +39,25 @@ namespace Om {
 class Request
 {
 public:
+	/** Construct a request.
+	 *
+	 * The lo_address must be duplicated here, as the address passed from
+	 * lo_message_get_source is not the client's responsibility, and liblo
+	 * does nasty things to it once the callback is completed.
+	 *
+	 * It would be nice to avoid this, it adds overhead to every single OSC
+	 * message received
+	 */
 	Request(lo_address source, int id)
-	: m_source(source),
+	: m_source(lo_address_new_from_url(lo_address_get_url(source))),
 	  m_id(id)
 	{
 	}
+
+	~Request()
+	{
+		lo_address_free(m_source);
+	}
 	
 	void respond_ok();
 	void respond_error(const string& msg);
@@ -49,6 +66,10 @@ public:
 	int        id()     { return m_id; }
 
 private:
+	// Prevent copies
+	Request(const Request& copy)            { exit(EXIT_FAILURE); }
+	Request& operator=(const Request& copy) { exit(EXIT_FAILURE); }
+
 	lo_address m_source;
 	int        m_id;
 };
