@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,7 @@ class OmObject;
 class SetMetadataEvent : public SlowEvent
 {
 public:
-	SetMetadataEvent(Request request, const string& path, const string& key, const string& value);
+	SetMetadataEvent(Request* request, const string& path, const string& key, const string& value);
 	
 	void prepare();
 	void execute(uint sample_offset);
