@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -22,10 +22,9 @@
 namespace Om {
 
 
-UnregisterClientEvent::UnregisterClientEvent(Request request, const string& host, const string& port)
+UnregisterClientEvent::UnregisterClientEvent(Request* request, const string& url)
 : SlowEvent(request),
-  m_host(host),
-  m_port(port),
+  m_url(url),
   m_success(false)
 {
 }
@@ -48,11 +47,13 @@ UnregisterClientEvent::execute(uint sample_offset)
 void
 UnregisterClientEvent::post_process()
 {
-	m_success = om->osc_receiver()->unregister_client(m_host, m_port);
-	if (m_success)
-		m_request.respond_ok();
-	else
-		m_request.respond_error("Unable to register client");
+	ClientRecord* cr = om->osc_receiver()->unregister_client(m_url);
+	if (cr != NULL) {
+		m_request->respond_ok();
+		delete cr;
+	} else {
+		m_request->respond_error("Unable to unregister client");
+	}
 }
 
 
