@@ -1,4 +1,4 @@
-/* This file is part of Om.  Copyright (C) 2004 Dave Robillard.
+/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
  * 
  * Om is free software; you can redistribute it and/or modify it under the
  * terms of the GNU General Public License as published by the Free Software
@@ -29,15 +29,14 @@ namespace Om {
 class UnregisterClientEvent : public SlowEvent
 {
 public:
-	UnregisterClientEvent(Request request, const string& host, const string& port);
-	
+	UnregisterClientEvent(Request* request, const string& url);
+
 	void prepare();
 	void execute(uint sample_offset);
 	void post_process();
 
 private:
-	string m_host;
-	string m_port;
+	string m_url;
 	bool   m_success;
 };
 
